from unittest import TestCase

from lxml import etree

from pcs.common.pacemaker.alert import (
    CibAlertDto,
    CibAlertRecipientDto,
    CibAlertSelectAttributeDto,
    CibAlertSelectDto,
)
from pcs.common.pacemaker.nvset import CibNvpairDto, CibNvsetDto
from pcs.common.reports import ReportItemSeverity as severities
from pcs.common.reports import codes as report_codes
from pcs.lib.cib import alert
from pcs.lib.cib.tools import IdProvider

from pcs_test.tools import fixture
from pcs_test.tools.assertions import (
    assert_raise_library_error,
    assert_report_item_list_equal,
    assert_xml_equal,
)


class UpdateOptionalAttributeTest(TestCase):
    # pylint: disable=protected-access
    def test_add(self):
        element = etree.Element("element")
        alert._update_optional_attribute(element, "attr", "value1")
        self.assertEqual(element.get("attr"), "value1")

    def test_update(self):
        element = etree.Element("element", attr="value")
        alert._update_optional_attribute(element, "attr", "value1")
        self.assertEqual(element.get("attr"), "value1")

    def test_remove(self):
        element = etree.Element("element", attr="value")
        alert._update_optional_attribute(element, "attr", "")
        self.assertTrue(element.get("attr") is None)


class ValidateRecipientValueIsUniqueTest(TestCase):
    # pylint: disable=protected-access
    def setUp(self):
        self.alert = etree.Element("alert", id="alert-1")
        self.recipient = etree.SubElement(
            self.alert, "recipient", id="rec-1", value="value1"
        )

    def test_is_unique_no_duplicity_allowed(self):
        report_list = alert._validate_recipient_value_is_unique(
            self.alert, "value2"
        )
        assert_report_item_list_equal(report_list, [])

    def test_same_recipient_no_duplicity_allowed(self):
        report_list = alert._validate_recipient_value_is_unique(
            self.alert, "value1", recipient_id="rec-1"
        )
        assert_report_item_list_equal(report_list, [])

    def test_same_recipient_duplicity_allowed(self):
        report_list = alert._validate_recipient_value_is_unique(
            self.alert,
            "value1",
            recipient_id="rec-1",
            allow_duplicity=True,
        )
        assert_report_item_list_equal(report_list, [])

    def test_not_unique_no_duplicity_allowed(self):
        report_list = alert._validate_recipient_value_is_unique(
            self.alert, "value1"
        )
        assert_report_item_list_equal(
            report_list,
            [
                fixture.error(
                    report_codes.CIB_ALERT_RECIPIENT_ALREADY_EXISTS,
                    force_code=report_codes.FORCE,
                    alert="alert-1",
                    recipient="value1",
                )
            ],
        )

    def test_is_unique_duplicity_allowed(self):
        report_list = alert._validate_recipient_value_is_unique(
            self.alert, "value2", allow_duplicity=True
        )
        assert_report_item_list_equal(report_list, [])

    def test_not_unique_duplicity_allowed(self):
        report_list = alert._validate_recipient_value_is_unique(
            self.alert, "value1", allow_duplicity=True
        )
        assert_report_item_list_equal(
            report_list,
            [
                fixture.warn(
                    report_codes.CIB_ALERT_RECIPIENT_ALREADY_EXISTS,
                    alert="alert-1",
                    recipient="value1",
                )
            ],
        )


class ValidateCreateAlert(TestCase):
    def setUp(self):
        self.tree = etree.XML(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert"/>
                    </alerts>
                </configuration>
            </cib>
            """
        )
        self.id_provider = IdProvider(self.tree)

    def test_empty_path(self):
        assert_report_item_list_equal(
            alert.validate_create_alert(self.id_provider, "", "alert2"),
            [
                fixture.error(
                    report_codes.REQUIRED_OPTIONS_ARE_MISSING,
                    option_names=["path"],
                    option_type=None,
                )
            ],
        )

    def test_invalid_id(self):
        assert_report_item_list_equal(
            alert.validate_create_alert(self.id_provider, "/path", "1alert"),
            [
                fixture.error(
                    report_codes.INVALID_ID_BAD_CHAR,
                    id="1alert",
                    id_description="alert-id",
                    invalid_character="1",
                    is_first_char=True,
                )
            ],
        )

    def test_id_exists(self):
        assert_report_item_list_equal(
            alert.validate_create_alert(self.id_provider, "/path", "alert"),
            [
                fixture.error(
                    report_codes.ID_ALREADY_EXISTS,
                    id="alert",
                ),
            ],
        )


class CreateAlert(TestCase):
    def setUp(self):
        self.tree = etree.XML(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert"/>
                    </alerts>
                </configuration>
            </cib>
            """
        )
        self.id_provider = IdProvider(self.tree)

    def test_no_alerts(self):
        # pylint: disable=no-self-use
        tree = etree.XML(
            """
            <cib>
                <configuration/>
            </cib>
            """
        )
        id_provider = IdProvider(tree)
        assert_xml_equal(
            '<alert id="my-alert" path="/test/path"/>',
            etree.tostring(
                alert.create_alert(tree, id_provider, "/test/path", "my-alert")
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="my-alert" path="/test/path"/>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(tree).decode(),
        )

    def test_alerts_exists(self):
        assert_xml_equal(
            '<alert id="my-alert" path="/test/path"/>',
            etree.tostring(
                alert.create_alert(
                    self.tree, self.id_provider, "/test/path", "my-alert"
                )
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert"/>
                        <alert id="my-alert" path="/test/path"/>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_alerts_exists_with_description(self):
        assert_xml_equal(
            '<alert id="my-alert" path="/test/path" description="nothing"/>',
            etree.tostring(
                alert.create_alert(
                    self.tree,
                    self.id_provider,
                    "/test/path",
                    "my-alert",
                    "nothing",
                )
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert"/>
                        <alert
                            id="my-alert"
                            path="/test/path"
                            description="nothing"
                        />
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_no_id(self):
        assert_xml_equal(
            '<alert id="alert-1" path="/test/path"/>',
            etree.tostring(
                alert.create_alert(self.tree, self.id_provider, "/test/path")
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert"/>
                        <alert id="alert-1" path="/test/path"/>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )


class UpdateAlertTest(TestCase):
    def setUp(self):
        self.tree = etree.XML(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path"/>
                        <alert id="alert1" path="/path1" description="nothing"/>
                    </alerts>
                </configuration>
            </cib>
            """
        )

    def test_update_path(self):
        assert_xml_equal(
            '<alert id="alert" path="/test/path"/>',
            etree.tostring(
                alert.update_alert(self.tree, "alert", "/test/path")
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/test/path"/>
                        <alert id="alert1" path="/path1" description="nothing"/>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_remove_path(self):
        assert_xml_equal(
            '<alert id="alert" path="/path"/>',
            etree.tostring(alert.update_alert(self.tree, "alert", "")).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path"/>
                        <alert id="alert1" path="/path1" description="nothing"/>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_update_description(self):
        assert_xml_equal(
            '<alert id="alert" path="/path" description="desc"/>',
            etree.tostring(
                alert.update_alert(self.tree, "alert", None, "desc")
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path" description="desc"/>
                        <alert id="alert1" path="/path1" description="nothing"/>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_remove_description(self):
        assert_xml_equal(
            '<alert id="alert1" path="/path1"/>',
            etree.tostring(
                alert.update_alert(self.tree, "alert1", None, "")
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path"/>
                        <alert id="alert1" path="/path1"/>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_id_not_exists(self):
        assert_raise_library_error(
            lambda: alert.update_alert(self.tree, "alert0", "/test"),
            (
                severities.ERROR,
                report_codes.ID_NOT_FOUND,
                {
                    "id": "alert0",
                    "expected_types": ["alert"],
                    "context_type": "alerts",
                    "context_id": "",
                },
                None,
            ),
        )


class RemoveAlertTest(TestCase):
    def setUp(self):
        self.tree = etree.XML(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path"/>
                        <alert id="alert-1" path="/next"/>
                    </alerts>
                </configuration>
            </cib>
            """
        )

    def test_success(self):
        alert.remove_alert(self.tree, "alert")
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert-1" path="/next"/>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_not_existing_id(self):
        assert_raise_library_error(
            lambda: alert.remove_alert(self.tree, "not-existing-id"),
            (
                severities.ERROR,
                report_codes.ID_NOT_FOUND,
                {
                    "id": "not-existing-id",
                    "expected_types": ["alert"],
                    "context_type": "alerts",
                    "context_id": "",
                },
                None,
            ),
        )


class ValidateAddRecipientTest(TestCase):
    def setUp(self):
        self.tree = etree.XML(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """
        )
        self.id_provider = IdProvider(self.tree)
        self.alert_el = self.tree.xpath(".//alert[@id='alert']")[0]

    def test_id_exists(self):
        report_list = alert.validate_add_recipient(
            self.id_provider,
            self.alert_el,
            "value1",
            "alert-recipient",
        )
        assert_report_item_list_equal(
            report_list,
            [
                fixture.error(
                    report_codes.ID_ALREADY_EXISTS,
                    id="alert-recipient",
                ),
            ],
        )

    def test_duplicity_of_value_not_allowed(self):
        report_list = alert.validate_add_recipient(
            self.id_provider,
            self.alert_el,
            "test_val",
        )
        assert_report_item_list_equal(
            report_list,
            [
                fixture.error(
                    report_codes.CIB_ALERT_RECIPIENT_ALREADY_EXISTS,
                    force_code=report_codes.FORCE,
                    alert="alert",
                    recipient="test_val",
                )
            ],
        )


class AddRecipientTest(TestCase):
    def setUp(self):
        self.tree = etree.XML(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """
        )
        self.id_provider = IdProvider(self.tree)
        self.alert_el = self.tree.xpath(".//alert[@id='alert']")[0]

    def test_with_id(self):
        assert_xml_equal(
            '<recipient id="my-recipient" value="value1"/>',
            etree.tostring(
                alert.add_recipient(
                    self.id_provider,
                    self.alert_el,
                    "value1",
                    "my-recipient",
                )
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                            <recipient id="my-recipient" value="value1"/>
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_without_id(self):
        assert_xml_equal(
            '<recipient id="alert-recipient-1" value="value1"/>',
            etree.tostring(
                alert.add_recipient(
                    self.id_provider,
                    self.alert_el,
                    "value1",
                )
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                            <recipient id="alert-recipient-1" value="value1"/>
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_with_description(self):
        assert_xml_equal(
            """
            <recipient
                id="alert-recipient-1"
                value="value1"
                description="desc"
            />
            """,
            etree.tostring(
                alert.add_recipient(
                    self.id_provider,
                    self.alert_el,
                    "value1",
                    description="desc",
                )
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                            <recipient
                                id="alert-recipient-1"
                                value="value1"
                                description="desc"
                            />
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )


class ValideteUpdateRecipientTest(TestCase):
    def setUp(self):
        self.tree = etree.XML(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                            <recipient
                                id="alert-recipient-1"
                                value="value1"
                                description="desc"
                            />
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """
        )
        self.recipient_el = self.tree.xpath(
            ".//recipient[@id='alert-recipient']"
        )[0]

    def test_update_same_value_no_duplicity_allowed(self):
        report_list = alert.validate_update_recipient(
            self.recipient_el,
            recipient_value="test_val",
        )
        assert_report_item_list_equal(report_list, [])

    def test_duplicity_of_value_not_allowed(self):
        report_list = alert.validate_update_recipient(
            self.recipient_el,
            "value1",
        )
        assert_report_item_list_equal(
            report_list,
            [
                fixture.error(
                    report_codes.CIB_ALERT_RECIPIENT_ALREADY_EXISTS,
                    force_code=report_codes.FORCE,
                    alert="alert",
                    recipient="value1",
                )
            ],
        )

    def test_duplicity_of_value_allowed(self):
        report_list = alert.validate_update_recipient(
            self.recipient_el,
            "value1",
            allow_same_value=True,
        )
        assert_report_item_list_equal(
            report_list,
            [
                fixture.warn(
                    report_codes.CIB_ALERT_RECIPIENT_ALREADY_EXISTS,
                    alert="alert",
                    recipient="value1",
                )
            ],
        )


class UpdateRecipientTest(TestCase):
    def setUp(self):
        self.tree = etree.XML(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                            <recipient
                                id="alert-recipient-1"
                                value="value1"
                                description="desc"
                            />
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """
        )
        self.recipient_el = self.tree.xpath(
            ".//recipient[@id='alert-recipient']"
        )[0]
        self.recipient1_el = self.tree.xpath(
            ".//recipient[@id='alert-recipient-1']"
        )[0]

    def test_update_value(self):
        assert_xml_equal(
            """
            <recipient id="alert-recipient" value="new_val"/>
            """,
            etree.tostring(
                alert.update_recipient(
                    self.recipient_el,
                    recipient_value="new_val",
                )
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="new_val"/>
                            <recipient
                                id="alert-recipient-1"
                                value="value1"
                                description="desc"
                            />
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_add_description(self):
        assert_xml_equal(
            """
            <recipient
                id="alert-recipient" value="test_val" description="description"
            />
            """,
            etree.tostring(
                alert.update_recipient(
                    self.recipient_el,
                    description="description",
                )
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient
                                id="alert-recipient"
                                value="test_val"
                                description="description"
                            />
                            <recipient
                                id="alert-recipient-1"
                                value="value1"
                                description="desc"
                            />
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_update_description(self):
        assert_xml_equal(
            """
            <recipient
                id="alert-recipient-1" value="value1" description="description"
            />
            """,
            etree.tostring(
                alert.update_recipient(
                    self.recipient1_el,
                    description="description",
                )
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                            <recipient
                                id="alert-recipient-1"
                                value="value1"
                                description="description"
                            />
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_remove_description(self):
        assert_xml_equal(
            """
                <recipient id="alert-recipient-1" value="value1"/>
            """,
            etree.tostring(
                alert.update_recipient(
                    self.recipient1_el,
                    description="",
                )
            ).decode(),
        )
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                            <recipient id="alert-recipient-1" value="value1"/>
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )


class RemoveRecipientTest(TestCase):
    def setUp(self):
        self.tree = etree.XML(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                            <recipient id="alert-recipient-2" value="val"/>
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """
        )

    def test_success(self):
        alert.remove_recipient(self.tree, "alert-recipient-2")
        assert_xml_equal(
            """
            <cib>
                <configuration>
                    <alerts>
                        <alert id="alert" path="/path">
                            <recipient id="alert-recipient" value="test_val"/>
                        </alert>
                    </alerts>
                </configuration>
            </cib>
            """,
            etree.tostring(self.tree).decode(),
        )

    def test_recipient_not_exists(self):
        assert_raise_library_error(
            lambda: alert.remove_recipient(self.tree, "recipient"),
            (
                severities.ERROR,
                report_codes.ID_NOT_FOUND,
                {
                    "id": "recipient",
                    "expected_types": ["recipient"],
                    "context_type": "alerts",
                    "context_id": "",
                },
                None,
            ),
        )


class CibMixin:
    maxDiff = None

    @staticmethod
    def get_cib():
        return etree.fromstring(
            """
            <cib><configuration>
            <resources />
            <alerts>
                <alert id="alert" path="/path">
                    <recipient id="alert-recipient" value="test_val">
                        <instance_attributes>
                            <nvpair
                                id="instance_attributes-name1-value1"
                                name="name1"
                                value="value1"
                            />
                            <nvpair
                                id="instance_attributes-name2-value2"
                                name="name2"
                                value="value2"
                            />
                        </instance_attributes>
                        <meta_attributes>
                            <nvpair id="meta_attributes-name3" name="name3"/>
                        </meta_attributes>
                    </recipient>
                    <recipient
                        id="alert-recipient-1" value="value1" description="desc"
                    />
                </alert>
                <alert id="alert1" path="/test/path" description="desc">
                    <instance_attributes>
                        <nvpair
                            id="alert1-name1-value1" name="name1" value="value1"
                        />
                        <nvpair
                            id="alert1-name2-value2" name="name2" value="value2"
                        />
                    </instance_attributes>
                    <meta_attributes>
                        <nvpair id="alert1-name3" name="name3"/>
                    </meta_attributes>
                </alert>
                <alert id="alert-select1" path="/path/select1">
                    <select>
                        <select_nodes />
                        <select_fencing />
                    </select>
                </alert>
                <alert id="alert-select2" path="/path/select2">
                    <select>
                        <select_resources />
                        <select_attributes>
                            <attribute id="alert-standby" name="standby" />
                            <attribute id="alert-shutdown" name="shutdown" />
                        </select_attributes>
                    </select>
                </alert>
                <alert id="alert-all" path="/path/all" description="all options">
                    <recipient id="alert-all-recipient" value="value-all"
                        description="all options recipient"
                    >
                        <instance_attributes>
                            <nvpair id="alert-all-recipient-ia"
                                name="all-iar1-name" value="all-iar1-value"
                            />
                        </instance_attributes>
                        <meta_attributes>
                            <nvpair id="alert-all-recipient-ma"
                                name="all-mar1-name" value="all-mar1-value" />
                        </meta_attributes>
                    </recipient>
                    <instance_attributes>
                        <nvpair id="alert-all-ia"
                            name="all-iaa1-name" value="all-iaa1-value"
                        />
                    </instance_attributes>
                    <meta_attributes>
                        <nvpair id="alert-all-ma"
                            name="all-maa1-name" value="all-maa1-value"
                        />
                    </meta_attributes>
                    <select>
                        <select_nodes />
                        <select_attributes />
                    </select>
                </alert>
            </alerts>
            </configuration></cib>
        """
        )


class AlertElToDto(CibMixin, TestCase):
    def setUp(self):
        self.cib = self.get_cib()
        self.tree = self.cib.find(".//alerts")

    def test_success(self):
        self.assertEqual(
            [
                alert.alert_el_to_dto(element)
                for element in alert.get_all_alert_elements(self.tree)
            ],
            [
                CibAlertDto(
                    id="alert",
                    path="/path",
                    description=None,
                    recipients=[
                        CibAlertRecipientDto(
                            id="alert-recipient",
                            value="test_val",
                            description=None,
                            meta_attributes=[
                                CibNvsetDto(
                                    id="",
                                    options={},
                                    rule=None,
                                    nvpairs=[
                                        CibNvpairDto(
                                            id="meta_attributes-name3",
                                            name="name3",
                                            value="",
                                        )
                                    ],
                                )
                            ],
                            instance_attributes=[
                                CibNvsetDto(
                                    id="",
                                    options={},
                                    rule=None,
                                    nvpairs=[
                                        CibNvpairDto(
                                            id="instance_attributes-name1-value1",
                                            name="name1",
                                            value="value1",
                                        ),
                                        CibNvpairDto(
                                            id="instance_attributes-name2-value2",
                                            name="name2",
                                            value="value2",
                                        ),
                                    ],
                                )
                            ],
                        ),
                        CibAlertRecipientDto(
                            id="alert-recipient-1",
                            value="value1",
                            description="desc",
                            meta_attributes=[],
                            instance_attributes=[],
                        ),
                    ],
                    select=None,
                    meta_attributes=[],
                    instance_attributes=[],
                ),
                CibAlertDto(
                    id="alert1",
                    path="/test/path",
                    description="desc",
                    recipients=[],
                    select=None,
                    meta_attributes=[
                        CibNvsetDto(
                            id="",
                            options={},
                            rule=None,
                            nvpairs=[
                                CibNvpairDto(
                                    id="alert1-name3", name="name3", value=""
                                )
                            ],
                        )
                    ],
                    instance_attributes=[
                        CibNvsetDto(
                            id="",
                            options={},
                            rule=None,
                            nvpairs=[
                                CibNvpairDto(
                                    id="alert1-name1-value1",
                                    name="name1",
                                    value="value1",
                                ),
                                CibNvpairDto(
                                    id="alert1-name2-value2",
                                    name="name2",
                                    value="value2",
                                ),
                            ],
                        )
                    ],
                ),
                CibAlertDto(
                    id="alert-select1",
                    path="/path/select1",
                    description=None,
                    recipients=[],
                    select=CibAlertSelectDto(
                        nodes=True,
                        fencing=True,
                        resources=False,
                        attributes=False,
                        attributes_select=[],
                    ),
                    meta_attributes=[],
                    instance_attributes=[],
                ),
                CibAlertDto(
                    id="alert-select2",
                    path="/path/select2",
                    description=None,
                    recipients=[],
                    select=CibAlertSelectDto(
                        nodes=False,
                        fencing=False,
                        resources=True,
                        attributes=True,
                        attributes_select=[
                            CibAlertSelectAttributeDto(
                                id="alert-standby", name="standby"
                            ),
                            CibAlertSelectAttributeDto(
                                id="alert-shutdown", name="shutdown"
                            ),
                        ],
                    ),
                    meta_attributes=[],
                    instance_attributes=[],
                ),
                CibAlertDto(
                    id="alert-all",
                    path="/path/all",
                    description="all options",
                    recipients=[
                        CibAlertRecipientDto(
                            id="alert-all-recipient",
                            value="value-all",
                            description="all options recipient",
                            meta_attributes=[
                                CibNvsetDto(
                                    id="",
                                    options={},
                                    rule=None,
                                    nvpairs=[
                                        CibNvpairDto(
                                            id="alert-all-recipient-ma",
                                            name="all-mar1-name",
                                            value="all-mar1-value",
                                        )
                                    ],
                                )
                            ],
                            instance_attributes=[
                                CibNvsetDto(
                                    id="",
                                    options={},
                                    rule=None,
                                    nvpairs=[
                                        CibNvpairDto(
                                            id="alert-all-recipient-ia",
                                            name="all-iar1-name",
                                            value="all-iar1-value",
                                        )
                                    ],
                                )
                            ],
                        )
                    ],
                    select=CibAlertSelectDto(
                        nodes=True,
                        fencing=False,
                        resources=False,
                        attributes=True,
                        attributes_select=[],
                    ),
                    meta_attributes=[
                        CibNvsetDto(
                            id="",
                            options={},
                            rule=None,
                            nvpairs=[
                                CibNvpairDto(
                                    id="alert-all-ma",
                                    name="all-maa1-name",
                                    value="all-maa1-value",
                                )
                            ],
                        )
                    ],
                    instance_attributes=[
                        CibNvsetDto(
                            id="",
                            options={},
                            rule=None,
                            nvpairs=[
                                CibNvpairDto(
                                    id="alert-all-ia",
                                    name="all-iaa1-name",
                                    value="all-iaa1-value",
                                )
                            ],
                        )
                    ],
                ),
            ],
        )


class GetAllRecipientsDictTest(TestCase):
    # covered by GetAllAlertsDictTest
    pass


class GetAllAlertsDictTest(CibMixin, TestCase):
    def setUp(self):
        self.cib = self.get_cib()

    def test_success(self):
        self.assertEqual(
            [
                {
                    "id": "alert",
                    "path": "/path",
                    "description": "",
                    "instance_attributes": [],
                    "meta_attributes": [],
                    "recipient_list": [
                        {
                            "id": "alert-recipient",
                            "value": "test_val",
                            "description": "",
                            "instance_attributes": [
                                {
                                    "id": "instance_attributes-name1-value1",
                                    "name": "name1",
                                    "value": "value1",
                                },
                                {
                                    "id": "instance_attributes-name2-value2",
                                    "name": "name2",
                                    "value": "value2",
                                },
                            ],
                            "meta_attributes": [
                                {
                                    "id": "meta_attributes-name3",
                                    "name": "name3",
                                    "value": None,
                                }
                            ],
                        },
                        {
                            "id": "alert-recipient-1",
                            "value": "value1",
                            "description": "desc",
                            "instance_attributes": [],
                            "meta_attributes": [],
                        },
                    ],
                },
                {
                    "id": "alert1",
                    "path": "/test/path",
                    "description": "desc",
                    "instance_attributes": [
                        {
                            "id": "alert1-name1-value1",
                            "name": "name1",
                            "value": "value1",
                        },
                        {
                            "id": "alert1-name2-value2",
                            "name": "name2",
                            "value": "value2",
                        },
                    ],
                    "meta_attributes": [
                        {"id": "alert1-name3", "name": "name3", "value": None}
                    ],
                    "recipient_list": [],
                },
                # select is not supported by the legacy export and it cannot be
                # added, because the export is passed to web ui and that may
                # not be able to cope with it
                {
                    "id": "alert-select1",
                    "path": "/path/select1",
                    "description": "",
                    "instance_attributes": [],
                    "meta_attributes": [],
                    "recipient_list": [],
                },
                {
                    "id": "alert-select2",
                    "path": "/path/select2",
                    "description": "",
                    "instance_attributes": [],
                    "meta_attributes": [],
                    "recipient_list": [],
                },
                {
                    "id": "alert-all",
                    "path": "/path/all",
                    "description": "all options",
                    "instance_attributes": [
                        {
                            "id": "alert-all-ia",
                            "name": "all-iaa1-name",
                            "value": "all-iaa1-value",
                        }
                    ],
                    "meta_attributes": [
                        {
                            "id": "alert-all-ma",
                            "name": "all-maa1-name",
                            "value": "all-maa1-value",
                        }
                    ],
                    "recipient_list": [
                        {
                            "id": "alert-all-recipient",
                            "value": "value-all",
                            "description": "all options recipient",
                            "instance_attributes": [
                                {
                                    "id": "alert-all-recipient-ia",
                                    "name": "all-iar1-name",
                                    "value": "all-iar1-value",
                                }
                            ],
                            "meta_attributes": [
                                {
                                    "id": "alert-all-recipient-ma",
                                    "name": "all-mar1-name",
                                    "value": "all-mar1-value",
                                }
                            ],
                        }
                    ],
                },
            ],
            alert.get_all_alerts_dict(self.cib),
        )



from unittest import (
    TestCase,
    mock,
)

from pcs import resource
from pcs.cli.common.parse_args import InputModifiers
from pcs.common import const
from pcs.common.str_tools import format_list
from pcs.lib.resource_agent import const as ra_const

from pcs_test.tier1.cib_resource.common import (
    ResourceTest,
    fixture_meta_attributes_not_validated_warning,
    fixture_meta_attributes_warning,
)
from pcs_test.tools.assertions import AssertPcsMixin
from pcs_test.tools.bin_mock import get_mock_settings
from pcs_test.tools.pcs_runner import PcsRunner

# pylint: disable=too-many-lines

ERRORS_HAVE_OCCURRED = (
    "Error: Errors have occurred, therefore pcs is unable to continue\n"
)


class Success(ResourceTest):
    msg_clone_without_meta = (
        "Deprecation Warning: Configuring clone meta attributes without "
        "specifying the 'meta' keyword after the 'clone' keyword is deprecated "
        "and will be removed in a future release. Specify --future to switch "
        "to the future behavior.\n"
    )

    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def test_base_create(self):
        self.assert_effect(
            "resource create R ocf:pcsmock:minimal --no-default-ops".split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_base_create_with_agent_name_including_systemd_instance(self):
        # crm_resource returns the same metadata for any systemd resource, no
        # matter if it exists or not
        self.assert_effect(
            "resource create R systemd:pcsmock@a:b --no-default-ops".split(),
            """<resources>
                <primitive class="systemd" id="R" type="pcsmock@a:b">
                    <operations>
                        <op id="R-monitor-interval-60s" interval="60s"
                            name="monitor" timeout="100s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_base_create_with_default_ops(self):
        self.assert_effect(
            "resource create R ocf:pcsmock:minimal".split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-migrate_from-interval-0s" interval="0s"
                            name="migrate_from" timeout="20s"
                        />
                        <op id="R-migrate_to-interval-0s" interval="0s"
                            name="migrate_to" timeout="20s"
                        />
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                        <op id="R-reload-interval-0s" interval="0s"
                            name="reload" timeout="20s"
                        />
                        <op id="R-reload-agent-interval-0s" interval="0s"
                            name="reload-agent" timeout="20s"
                        />
                        <op id="R-start-interval-0s" interval="0s" name="start"
                            timeout="20s"
                        />
                        <op id="R-stop-interval-0s" interval="0s" name="stop"
                            timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_create_with_options(self):
        self.assert_effect(
            (
                "resource create --no-default-ops R ocf:pcsmock:params "
                "mandatory=mandat optional=opti"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="params">
                    <instance_attributes id="R-instance_attributes">
                        <nvpair id="R-instance_attributes-mandatory"
                            name="mandatory" value="mandat"
                        />
                        <nvpair id="R-instance_attributes-optional"
                            name="optional" value="opti"
                        />
                    </instance_attributes>
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_create_with_trace_options(self):
        # trace_ra and trace_file options are not defined in metadata but they
        # are allowed for all ocf:heartbeat and ocf:pacemaker agents. This test
        # checks it is possible to set them without --force.
        self.assert_effect(
            (
                "resource create --no-default-ops R ocf:heartbeat:pcsMock "
                "trace_ra=1 trace_file=/root/trace"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat"
                    type="pcsMock"
                >
                    <instance_attributes id="R-instance_attributes">
                        <nvpair id="R-instance_attributes-trace_file"
                            name="trace_file" value="/root/trace"
                        />
                        <nvpair id="R-instance_attributes-trace_ra"
                            name="trace_ra" value="1"
                        />
                    </instance_attributes>
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_create_with_options_and_operations(self):
        self.assert_effect(
            (
                "resource create --no-default-ops R ocf:pcsmock:params "
                "mandatory=mandat optional=opti op monitor interval=30s"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="params">
                    <instance_attributes id="R-instance_attributes">
                        <nvpair id="R-instance_attributes-mandatory"
                            name="mandatory" value="mandat"
                        />
                        <nvpair id="R-instance_attributes-optional"
                            name="optional" value="opti"
                        />
                    </instance_attributes>
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_create_disabled(self):
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                "--disabled"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <meta_attributes id="R-meta_attributes">
                        <nvpair id="R-meta_attributes-target-role"
                            name="target-role" value="Stopped"
                        />
                    </meta_attributes>
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_with_clone(self):
        self.assert_effect(
            "resource create R ocf:pcsmock:minimal --no-default-ops clone".split(),
            """<resources>
                <clone id="R-clone">
                    <primitive class="ocf" id="R" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                </clone>
            </resources>""",
        )

    def test_with_custom_clone_id(self):
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops clone "
                "CustomId"
            ).split(),
            """<resources>
                <clone id="CustomId">
                    <primitive class="ocf" id="R" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                </clone>
            </resources>""",
        )

    def test_with_clone_options(self):
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops clone "
                "notify=true"
            ).split(),
            """<resources>
                <clone id="R-clone">
                    <primitive class="ocf" id="R" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <meta_attributes id="R-clone-meta_attributes">
                        <nvpair id="R-clone-meta_attributes-notify"
                            name="notify" value="true"
                        />
                    </meta_attributes>
                </clone>
            </resources>""",
            stderr_full=self.msg_clone_without_meta
            + fixture_meta_attributes_not_validated_warning(["clone"]),
        )

    def test_create_with_options_and_meta(self):
        self.assert_effect(
            (
                "resource create --no-default-ops R ocf:pcsmock:params "
                "mandatory=mandat optional=opti meta is-managed=false"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="params">
                    <instance_attributes id="R-instance_attributes">
                        <nvpair id="R-instance_attributes-mandatory"
                            name="mandatory" value="mandat"
                        />
                        <nvpair id="R-instance_attributes-optional"
                            name="optional" value="opti"
                        />
                    </instance_attributes>
                    <meta_attributes id="R-meta_attributes">
                        <nvpair id="R-meta_attributes-is-managed"
                            name="is-managed" value="false"
                        />
                    </meta_attributes>
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_create_with_unknown_meta(self):
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops meta "
                "unknown_meta=unknown_value"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <meta_attributes id="R-meta_attributes">
                        <nvpair id="R-meta_attributes-unknown_meta"
                            name="unknown_meta" value="unknown_value"></nvpair>
                    </meta_attributes>
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
            stderr_full=fixture_meta_attributes_warning(
                ["unknown_meta"], ra_const.PRIMITIVE_META
            ),
        )


class SuccessOperations(ResourceTest):
    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def test_create_with_operations(self):
        self.assert_effect(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor interval=30s"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_multiple_op_keyword(self):
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                "op monitor interval=30s op monitor interval=20s"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor"
                        />
                        <op id="R-monitor-interval-20s" interval="20s"
                            name="monitor"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_multiple_operations_same_op_keyword(self):
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                "op monitor interval=30s monitor interval=20s"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor"
                        />
                        <op id="R-monitor-interval-20s" interval="20s"
                            name="monitor"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_multiple_op_options_for_same_action(self):
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                "op monitor interval=30s timeout=20s"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_op_with_OCF_CHECK_LEVEL(self):
        # pylint: disable=invalid-name
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                "op monitor interval=30s timeout=20s OCF_CHECK_LEVEL=1"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor" timeout="20s"
                        >
                            <instance_attributes
                                id="R-monitor-interval-30s-instance_attributes"
                            >
                                <nvpair
                                    id="R-monitor-interval-30s-"""
            + 'instance_attributes-OCF_CHECK_LEVEL"'
            + """
                                    name="OCF_CHECK_LEVEL" value="1"
                                />
                            </instance_attributes>
                        </op>
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_default_ops_only(self):
        self.assert_effect(
            "resource create R ocf:pcsmock:minimal".split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-migrate_from-interval-0s" interval="0s"
                            name="migrate_from" timeout="20s"
                        />
                        <op id="R-migrate_to-interval-0s" interval="0s"
                            name="migrate_to" timeout="20s"
                        />
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                        <op id="R-reload-interval-0s" interval="0s"
                            name="reload" timeout="20s"
                        />
                        <op id="R-reload-agent-interval-0s" interval="0s"
                            name="reload-agent" timeout="20s"
                        />
                        <op id="R-start-interval-0s" interval="0s" name="start"
                            timeout="20s"
                        />
                        <op id="R-stop-interval-0s" interval="0s" name="stop"
                            timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_merging_default_ops_explicitly_specified(self):
        self.assert_effect(
            "resource create R ocf:pcsmock:minimal op start timeout=200".split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-migrate_from-interval-0s" interval="0s"
                            name="migrate_from" timeout="20s"
                        />
                        <op id="R-migrate_to-interval-0s" interval="0s"
                            name="migrate_to" timeout="20s"
                        />
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                        <op id="R-reload-interval-0s" interval="0s"
                            name="reload" timeout="20s"
                        />
                        <op id="R-reload-agent-interval-0s" interval="0s"
                            name="reload-agent" timeout="20s"
                        />
                        <op id="R-start-interval-0s" interval="0s" name="start"
                            timeout="200"
                        />
                        <op id="R-stop-interval-0s" interval="0s" name="stop"
                            timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_completing_monitor_operation(self):
        self.assert_effect(
            "resource create --no-default-ops R ocf:pcsmock:minimal".split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
        )

    def test_adapt_second_op_interval(self):
        self.assert_effect(
            "resource create R ocf:pcsmock:duplicate_monitor".split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock"
                    type="duplicate_monitor"
                >
                    <operations>
                        <op id="R-demote-interval-0s" interval="0s"
                            name="demote" timeout="10s"
                        />
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" role="Master" timeout="20s"
                        />
                        <op id="R-monitor-interval-11" interval="11"
                            name="monitor" role="Slave" timeout="20s"
                        />
                        <op id="R-notify-interval-0s" interval="0s"
                            name="notify" timeout="5s"
                        />
                        <op id="R-promote-interval-0s" interval="0s"
                            name="promote" timeout="10s"
                        />
                        <op id="R-reload-agent-interval-0s" interval="0s"
                            name="reload-agent" timeout="10s"
                        />
                        <op id="R-start-interval-0s" interval="0s" name="start"
                            timeout="20s"
                        />
                        <op id="R-stop-interval-0s" interval="0s" name="stop"
                            timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
            stderr_full=(
                "Warning: changing a monitor operation interval from 10s to 11 to"
                " make the operation unique\n"
            ),
        )

    def test_warn_on_forced_unknown_operation(self):
        self.assert_effect(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitro interval=30s --force"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                        <op id="R-monitro-interval-30s" interval="30s"
                            name="monitro"
                        />
                    </operations>
                </primitive>
            </resources>""",
            stderr_full=(
                "Warning: 'monitro' is not a valid operation name value, use "
                "'meta-data', 'migrate_from', 'migrate_to', 'monitor', "
                "'reload', 'reload-agent', 'start', 'stop', 'validate-all'\n"
            ),
        )

    def test_op_id(self):
        self.assert_effect(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor interval=30s id=abcd"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <operations>
                        <op id="abcd" interval="30s" name="monitor" />
                    </operations>
                </primitive>
            </resources>""",
        )


class SuccessNewParser(ResourceTest):
    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def test_primitive_meta(self):
        self.assert_effect(
            "resource create R ocf:pcsmock:minimal meta a=b --no-default-ops --future".split(),
            """<resources>
                <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                    <meta_attributes id="R-meta_attributes">
                        <nvpair id="R-meta_attributes-a" name="a" value="b"/>
                    </meta_attributes>
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
            stderr_full=fixture_meta_attributes_warning(
                ["a"], ra_const.PRIMITIVE_META
            ),
        )

    def test_clone_meta(self):
        self.assert_effect(
            "resource create R ocf:pcsmock:minimal clone meta a=b --no-default-ops --future".split(),
            """<resources>
                <clone id="R-clone">
                    <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <meta_attributes id="R-clone-meta_attributes">
                        <nvpair id="R-clone-meta_attributes-a" name="a" value="b"/>
                    </meta_attributes>
                </clone>
            </resources>""",
            stderr_full=fixture_meta_attributes_not_validated_warning(
                ["clone"]
            ),
        )

    def test_primitive_and_clone_meta(self):
        self.assert_effect(
            "resource create R ocf:pcsmock:minimal meta a=b clone meta c=d --no-default-ops --future".split(),
            """<resources>
                <clone id="R-clone">
                    <primitive class="ocf" id="R" provider="pcsmock" type="minimal">
                        <meta_attributes id="R-meta_attributes">
                            <nvpair id="R-meta_attributes-a" name="a" value="b"/>
                        </meta_attributes>
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <meta_attributes id="R-clone-meta_attributes">
                        <nvpair id="R-clone-meta_attributes-c" name="c" value="d"/>
                    </meta_attributes>
                </clone>
            </resources>""",
            stderr_full=fixture_meta_attributes_not_validated_warning(["clone"])
            + fixture_meta_attributes_warning(["a"], ra_const.PRIMITIVE_META),
        )


class SuccessGroup(ResourceTest):
    FUTURE = ""
    GROUP = "--group"
    AFTER = "--after"
    BEFORE = "--before"
    DEPRECATED_GROUP = (
        "Deprecation Warning: Using '--group' is deprecated and will be "
        "replaced with 'group' in a future release. Specify --future to switch "
        "to the future behavior.\n"
    )
    DEPRECATED_AFTER = (
        "Deprecation Warning: Using '--after' is deprecated and will be "
        "replaced with 'after' in a future release. Specify --future to switch "
        "to the future behavior.\n"
    )
    DEPRECATED_BEFORE = (
        "Deprecation Warning: Using '--before' is deprecated and will be "
        "replaced with 'before' in a future release. Specify --future to switch "
        "to the future behavior.\n"
    )

    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def test_with_group(self):
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                f"{self.GROUP} G {self.FUTURE}"
            ).split(),
            """<resources>
                <group id="G">
                    <primitive class="ocf" id="R" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                </group>
            </resources>""",
            stderr_full=self.DEPRECATED_GROUP,
        )

    def test_with_existing_group(self):
        self.assert_pcs_success(
            (
                "resource create R0 ocf:pcsmock:minimal --no-default-ops "
                f"{self.GROUP} G {self.FUTURE}"
            ).split(),
            stderr_full=self.DEPRECATED_GROUP,
        )
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                f"{self.GROUP} G {self.FUTURE}"
            ).split(),
            """<resources>
                <group id="G">
                    <primitive class="ocf" id="R0" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R0-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="R" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                </group>
            </resources>""",
            stderr_full=self.DEPRECATED_GROUP,
        )

    def test_with_group_with_after(self):
        self.assert_pcs_success(
            (
                "resource create R0 ocf:pcsmock:minimal --no-default-ops "
                f"{self.GROUP} G {self.FUTURE}"
            ).split(),
            stderr_full=self.DEPRECATED_GROUP,
        )
        self.assert_pcs_success(
            (
                "resource create R1 ocf:pcsmock:minimal --no-default-ops "
                f"{self.GROUP} G {self.FUTURE}"
            ).split(),
            stderr_full=self.DEPRECATED_GROUP,
        )
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                f"{self.GROUP} G {self.AFTER} R0 {self.FUTURE}"
            ).split(),
            """<resources>
                <group id="G">
                    <primitive class="ocf" id="R0" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R0-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="R" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="R1" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R1-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                </group>
            </resources>""",
            stderr_full=self.DEPRECATED_GROUP + self.DEPRECATED_AFTER,
        )
        self.assert_effect(
            (
                "resource create Rx ocf:pcsmock:minimal --no-default-ops "
                f"{self.GROUP} G {self.AFTER} R1 {self.FUTURE}"
            ).split(),
            """<resources>
                <group id="G">
                    <primitive class="ocf" id="R0" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R0-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="R" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="R1" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R1-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="Rx" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="Rx-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                </group>
            </resources>""",
            stderr_full=self.DEPRECATED_GROUP + self.DEPRECATED_AFTER,
        )

    def test_with_group_with_before(self):
        self.assert_pcs_success(
            (
                "resource create R0 ocf:pcsmock:minimal --no-default-ops "
                f"{self.GROUP} G {self.FUTURE}"
            ).split(),
            stderr_full=self.DEPRECATED_GROUP,
        )
        self.assert_effect(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                f"{self.GROUP} G {self.BEFORE} R0 {self.FUTURE}"
            ).split(),
            """<resources>
                <group id="G">
                    <primitive class="ocf" id="R" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="R0" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R0-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                </group>
            </resources>""",
            stderr_full=self.DEPRECATED_GROUP + self.DEPRECATED_BEFORE,
        )


class SuccessGroupFuture(SuccessGroup):
    FUTURE = "--future"
    GROUP = "group"
    AFTER = "after"
    BEFORE = "before"
    DEPRECATED_GROUP = DEPRECATED_AFTER = DEPRECATED_BEFORE = ""


class SuccessClone(ResourceTest):
    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def test_clone_places_disabled_correctly(self):
        self.assert_effect(
            "resource create R ocf:pcsmock:minimal clone --disabled".split(),
            """<resources>
                <clone id="R-clone">
                    <meta_attributes id="R-clone-meta_attributes">
                        <nvpair id="R-clone-meta_attributes-target-role"
                            name="target-role" value="Stopped"
                        />
                    </meta_attributes>
                    <primitive class="ocf" id="R" provider="pcsmock"
                        type="minimal"
                    >
                        <operations>
                            <op id="R-migrate_from-interval-0s" interval="0s"
                                name="migrate_from" timeout="20s"
                            />
                            <op id="R-migrate_to-interval-0s" interval="0s"
                                name="migrate_to" timeout="20s"
                            />
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                            <op id="R-reload-interval-0s" interval="0s"
                                name="reload" timeout="20s"
                            />
                            <op id="R-reload-agent-interval-0s" interval="0s"
                                name="reload-agent" timeout="20s"
                            />
                            <op id="R-start-interval-0s" interval="0s"
                                name="start" timeout="20s"
                            />
                            <op id="R-stop-interval-0s" interval="0s"
                                name="stop" timeout="20s"
                            />
                        </operations>
                    </primitive>
                </clone>
            </resources>""",
        )


class Promotable(TestCase, AssertPcsMixin):
    msg_promotable_without_meta = (
        "Deprecation Warning: Configuring promotable meta attributes without "
        "specifying the 'meta' keyword after the 'promotable' keyword is deprecated "
        "and will be removed in a future release. Specify --future to switch "
        "to the future behavior.\n"
    )

    def setUp(self):
        self.lib = mock.Mock(spec_set=["resource"])
        self.resource = mock.Mock(spec_set=["create_as_clone"])
        self.lib.resource = self.resource
        # used for tests where code does not even call lib, so cib is not needed
        self.pcs_runner = PcsRunner(cib_file=None)
        self.pcs_runner.mock_settings = get_mock_settings()

    @staticmethod
    def fixture_options(
        *,
        allow_absent_agent=False,
        allow_invalid_instance_attributes=False,
        allow_invalid_operation=False,
        allow_not_suitable_command=False,
        ensure_disabled=False,
        use_default_operations=True,
        wait=False,
        enable_agent_self_validation=False,
    ):
        # pylint: disable=unused-argument
        return locals()

    @mock.patch("pcs.cli.reports.output.print_to_stderr")
    def test_alias_for_clone(self, mock_print_to_stderr):
        resource.resource_create(
            self.lib,
            ["R", "ocf:pcsmock:stateful", "promotable", "a=b", "c=d"],
            InputModifiers({}),
        )
        self.resource.create_as_clone.assert_called_once_with(
            "R",
            "ocf:pcsmock:stateful",
            [],
            {},
            {},
            {"a": "b", "c": "d", "promotable": "true"},
            clone_id=None,
            allow_incompatible_clone_meta_attributes=False,
            **self.fixture_options(),
        )
        mock_print_to_stderr.assert_called_once_with(
            "Deprecation Warning: Configuring promotable meta attributes "
            "without specifying the 'meta' keyword after the 'promotable' "
            "keyword is deprecated and will be removed in a future release. "
            "Specify --future to switch to the future behavior."
        )

    def test_fail_on_promotable(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:stateful promotable promotable=a"
            ).split(),
            (
                self.msg_promotable_without_meta
                + "Error: you cannot specify both promotable option and promotable "
                "keyword\n"
            ),
        )

    def test_fail_on_promotable_true(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:stateful promotable "
                "promotable=true"
            ).split(),
            (
                self.msg_promotable_without_meta
                + "Error: you cannot specify both promotable option and promotable "
                "keyword\n"
            ),
        )

    def test_fail_on_promotable_false(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:stateful promotable "
                "promotable=false"
            ).split(),
            (
                self.msg_promotable_without_meta
                + "Error: you cannot specify both promotable option and promotable "
                "keyword\n"
            ),
        )


class Bundle(ResourceTest):
    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def fixture_primitive(self, name, bundle=None):
        if bundle:
            self.assert_pcs_success(
                [
                    "resource",
                    "create",
                    name,
                    "ocf:pcsmock:minimal",
                    "bundle",
                    bundle,
                ]
            )
        else:
            self.assert_pcs_success(
                ["resource", "create", name, "ocf:pcsmock:minimal"]
            )

    def fixture_bundle(self, name):
        self.assert_pcs_success(
            [
                "resource",
                "bundle",
                "create",
                name,
                "container",
                "docker",
                "image=pcs:test",
                "network",
                "control-port=1234",
            ],
        )

    def test_bundle_id_not_specified(self):
        self.assert_pcs_fail(
            "resource create R ocf:pcsmock:minimal --no-default-ops bundle".split(),
            "Error: you have to specify exactly one bundle\n",
        )

    def test_bundle_id_is_not_bundle(self):
        self.fixture_primitive("R1")
        self.assert_pcs_fail(
            "resource create R2 ocf:pcsmock:minimal bundle R1".split(),
            "Error: 'R1' is not a bundle\n",
        )

    def test_bundle_id_does_not_exist(self):
        self.assert_pcs_fail(
            "resource create R1 ocf:pcsmock:minimal bundle B".split(),
            "Error: bundle 'B' does not exist\n",
        )

    def test_primitive_already_in_bundle(self):
        self.fixture_bundle("B")
        self.fixture_primitive("R1", bundle="B")
        self.assert_pcs_fail(
            (
                "resource create R2 ocf:pcsmock:minimal --no-default-ops "
                "bundle B"
            ).split(),
            (
                "Error: bundle 'B' already contains resource 'R1', a bundle "
                "may contain at most one resource\n"
            ),
        )

    def test_success(self):
        self.fixture_bundle("B")
        self.assert_effect(
            (
                "resource create R1 ocf:pcsmock:minimal --no-default-ops "
                "bundle B"
            ).split(),
            """
                <resources>
                    <bundle id="B">
                        <docker image="pcs:test" />
                        <network control-port="1234"/>
                        <primitive class="ocf" id="R1" provider="pcsmock"
                            type="minimal"
                        >
                            <operations>
                                <op id="R1-monitor-interval-10s" interval="10s"
                                    name="monitor" timeout="20s"
                                />
                            </operations>
                        </primitive>
                    </bundle>
                </resources>
            """,
        )


class FailOrWarnGroupCloneBundleCombination(ResourceTest):
    FUTURE = ""
    GROUP = "--group"
    AFTER = "--after"
    BEFORE = "--before"
    DEPRECATED_GROUP = (
        "Deprecation Warning: Using '--group' is deprecated and will be "
        "replaced with 'group' in a future release. Specify --future to switch "
        "to the future behavior.\n"
    )

    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def test_error_group_clone_combination(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                f"clone {self.GROUP} G {self.FUTURE}"
            ).split(),
            (
                self.DEPRECATED_GROUP
                + "Error: you can specify only one of clone, promotable, bundle "
                f"or {self.GROUP}\n"
            ),
        )

    def test_error_bundle_clone_combination(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                f"clone bundle bundle_id {self.FUTURE}"
            ).split(),
            (
                "Error: you can specify only one of clone, promotable, bundle "
                f"or {self.GROUP}\n"
            ),
        )

    def test_error_bundle_group_combination(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:minimal --no-default-ops "
                f"{self.GROUP} G bundle bundle_id {self.FUTURE}"
            ).split(),
            (
                self.DEPRECATED_GROUP
                + "Error: you can specify only one of clone, promotable, bundle "
                f"or {self.GROUP}\n"
            ),
        )


class FailOrWarnGroupCloneBundleCombinationFuture(
    FailOrWarnGroupCloneBundleCombination
):
    FUTURE = "--future"
    GROUP = "group"
    AFTER = "after"
    BEFORE = "before"
    DEPRECATED_GROUP = ""


class FailOrWarn(ResourceTest):
    # pylint: disable=too-many-public-methods
    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def test_fail_when_nonexisting_agent(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:NoExisting".split(),
            stderr_full=(
                "Error: Agent 'ocf:heartbeat:NoExisting' is not installed or "
                "does not provide valid metadata: "
                "pcs mock error message: unable to load agent metadata, "
                "use --force to override\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_warn_when_forcing_nonexistent_agent(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:NoExisting --force".split(),
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat"
                    type="NoExisting"
                >
                    <operations>
                        <op id="R-monitor-interval-60s" interval="60s"
                            name="monitor"
                        />
                    </operations>
                </primitive>
            </resources>""",
            stderr_full=(
                "Warning: Agent 'ocf:heartbeat:NoExisting' is not installed or "
                "does not provide valid metadata: "
                "pcs mock error message: unable to load agent metadata\n"
            ),
        )

    def test_fail_on_invalid_resource_agent_name(self):
        self.assert_pcs_fail(
            "resource create R invalid_agent_name".split(),
            "Error: Unable to find agent 'invalid_agent_name', try specifying"
            " its full name\n" + ERRORS_HAVE_OCCURRED,
        )

    def test_fail_on_invalid_resource_agent_name_even_if_forced(self):
        self.assert_pcs_fail(
            "resource create R invalid_agent_name --force".split(),
            "Error: Unable to find agent 'invalid_agent_name', try specifying"
            " its full name\n" + ERRORS_HAVE_OCCURRED,
        )

    def test_fail_when_invalid_agent(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat: --force".split(),
            "Error: Invalid resource agent name 'ocf:heartbeat:'. Use"
            " standard:provider:type when standard is 'ocf' or"
            " standard:type otherwise. List of standards and providers can"
            " be obtained by using commands 'pcs resource standards' and"
            " 'pcs resource providers'.\n" + ERRORS_HAVE_OCCURRED,
        )

    def test_vail_when_agent_class_is_not_allowed(self):
        self.assert_pcs_fail(
            "resource create R invalid:Dummy --force".split(),
            "Error: Invalid resource agent name 'invalid:Dummy'. Use"
            " standard:provider:type when standard is 'ocf' or"
            " standard:type otherwise. List of standards and providers can"
            " be obtained by using commands 'pcs resource standards' and"
            " 'pcs resource providers'.\n" + ERRORS_HAVE_OCCURRED,
        )

    def test_fail_when_missing_provider_with_ocf_resource_agent(self):
        self.assert_pcs_fail(
            "resource create R ocf:Dummy".split(),
            "Error: Invalid resource agent name 'ocf:Dummy'. Use"
            " standard:provider:type when standard is 'ocf' or"
            " standard:type otherwise. List of standards and providers can"
            " be obtained by using commands 'pcs resource standards' and"
            " 'pcs resource providers'.\n" + ERRORS_HAVE_OCCURRED,
        )

    def test_fail_when_provider_appear_with_non_ocf_resource_agent(self):
        self.assert_pcs_fail(
            "resource create R lsb:provider:Dummy --force".split(),
            "Error: Invalid resource agent name 'lsb:provider:Dummy'. Use"
            " standard:provider:type when standard is 'ocf' or"
            " standard:type otherwise. List of standards and providers can"
            " be obtained by using commands 'pcs resource standards' and"
            " 'pcs resource providers'.\n" + ERRORS_HAVE_OCCURRED,
        )

    def test_print_info_about_agent_completion(self):
        self.assert_pcs_success(
            "resource create R camelcase".split(),
            stderr_full=(
                "Assumed agent name 'ocf:pcsmock:CamelCase' "
                "(deduced from 'camelcase')\n"
            ),
        )

    def test_fail_for_unambiguous_agent(self):
        self.assert_pcs_fail(
            "resource create R pcsmock".split(),
            "Error: Multiple agents match 'pcsmock', please specify full name:"
            " 'ocf:heartbeat:pcsMock' or 'ocf:pacemaker:pcsMock'\n"
            + ERRORS_HAVE_OCCURRED,
        )

    def test_for_options_not_matching_resource_agent(self):
        self.assert_pcs_fail(
            "resource create R ocf:pcsmock:params a=b mandatory=x c=d".split(),
            "Error: invalid resource options: 'a', 'c', allowed options are: "
            "'advanced', 'enum', 'mandatory', 'optional', 'unique1', 'unique2'"
            ", use --force to override\n" + ERRORS_HAVE_OCCURRED,
        )

    def test_for_missing_options_of_resource_agent(self):
        self.assert_pcs_fail(
            "resource create --no-default-ops R params".split(),
            (
                "Assumed agent name 'ocf:pcsmock:params' (deduced from"
                " 'params')\n"
                "Error: required resource option 'mandatory' is missing,"
                " use --force to override\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_on_invalid_resource_id(self):
        self.assert_pcs_fail(
            "resource create #R ocf:pcsmock:minimal".split(),
            "Error: invalid resource name '#R',"
            " '#' is not a valid first character for a resource name\n",
        )

    def test_fail_on_existing_resource_id(self):
        self.assert_pcs_success("resource create R ocf:pcsmock:minimal".split())
        self.assert_pcs_fail(
            "resource create R ocf:pcsmock:minimal".split(),
            "Error: 'R' already exists\n",
        )

    def test_fail_on_invalid_operation_id(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:minimal "
                "op monitor interval=30 id=#O"
            ).split(),
            (
                "Error: invalid operation id '#O',"
                " '#' is not a valid first character for a operation id\n"
                + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_on_existing_operation_id(self):
        self.assert_pcs_success("resource create R ocf:pcsmock:minimal".split())
        self.assert_pcs_fail(
            (
                "resource create S ocf:pcsmock:minimal "
                "op monitor interval=30 id=R"
            ).split(),
            "Error: 'R' already exists\n",
        )

    def test_fail_on_duplicate_operation_id(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:minimal "
                "op monitor interval=30 id=O op monitor interval=60 id=O"
            ).split(),
            "Error: 'O' already exists\n",
        )

    def test_fail_on_resource_id_same_as_operation_id(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:minimal "
                "op monitor interval=30 id=R"
            ).split(),
            "Error: 'R' already exists\n",
        )

    def test_fail_on_unknown_operation(self):
        self.assert_pcs_fail(
            "resource create R ocf:pcsmock:minimal op monitro interval=100".split(),
            (
                "Error: 'monitro' is not a valid operation name value, use"
                " 'meta-data', 'migrate_from', 'migrate_to', 'monitor',"
                " 'reload', 'reload-agent', 'start', 'stop', 'validate-all', "
                "use --force to override\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_on_ambiguous_value_of_option(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:minimal "
                "op monitor timeout=10 timeout=20"
            ).split(),
            "Error: duplicate option 'timeout' with different values '10' and"
            " '20'\n",
        )

    def test_unique_err(self):
        self.assert_pcs_success(
            "resource create R1 ocf:pcsmock:unique state=1".split()
        )
        self.assert_pcs_fail(
            "resource create R2 ocf:pcsmock:unique state=1".split(),
            (
                "Error: Value '1' of option 'state' is not unique across "
                "'ocf:pcsmock:unique' resources. Following resources are "
                "configured with the same value of the instance attribute: "
                "'R1', use --force to override\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_unique_multiple_resources_warn_and_err(self):
        self.assert_pcs_success(
            "resource create R1 ocf:pcsmock:unique state=1".split()
        )
        self.assert_pcs_success(
            "resource create R2 ocf:pcsmock:unique state=1 --force".split(),
            stderr_full=(
                "Warning: Value '1' of option 'state' is not unique across "
                "'ocf:pcsmock:unique' resources. Following resources are "
                "configured with the same value of the instance attribute: 'R1'\n"
            ),
        )
        self.assert_pcs_success(
            "resource create R3 ocf:pcsmock:unique state=1 --force".split(),
            stderr_full=(
                "Warning: Value '1' of option 'state' is not unique across "
                "'ocf:pcsmock:unique' resources. Following resources are "
                "configured with the same value of the instance attribute: 'R1', "
                "'R2'\n"
            ),
        )
        self.assert_pcs_fail(
            "resource create R4 ocf:pcsmock:unique state=1".split(),
            (
                "Error: Value '1' of option 'state' is not unique across "
                "'ocf:pcsmock:unique' resources. Following resources are "
                "configured with the same value of the instance attribute: "
                "'R1', 'R2', 'R3', use --force to override\n"
                + ERRORS_HAVE_OCCURRED
            ),
        )


class FailOrWarnOp(ResourceTest):
    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def test_fail_empty(self):
        self.assert_pcs_fail(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op meta is-managed=false"
            ).split(),
            "Error: When using 'op' you must specify an operation name and at"
            " least one option\n",
        )

    def test_fail_only_name_without_any_option(self):
        self.assert_pcs_fail(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor meta is-managed=false"
            ).split(),
            "Error: When using 'op' you must specify an operation name and at"
            " least one option\n",
        )

    def test_fail_duplicit(self):
        self.assert_pcs_fail(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor interval=1h monitor interval=3600sec "
                "monitor interval=1min monitor interval=60s"
            ).split(),
            (
                "Error: multiple specification of the same operation with the"
                " same interval:\n"
                "monitor with intervals 1h, 3600sec\n"
                "monitor with intervals 1min, 60s\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_invalid_first_action(self):
        self.assert_pcs_fail(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op mo=nitor interval=1min"
            ).split(),
            "Error: When using 'op' you must specify an operation name after"
            " 'op'\n",
        )

    def test_fail_invalid_option(self):
        self.assert_pcs_fail(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor interval=1min moni=tor timeout=80s"
            ).split(),
            "Error: invalid resource operation option 'moni', allowed options"
            " are: 'OCF_CHECK_LEVEL', 'description', 'enabled', 'id',"
            " 'interval', 'interval-origin', 'name', 'on-fail',"
            " 'record-pending', 'role', 'start-delay', 'timeout'\n"
            + ERRORS_HAVE_OCCURRED,
        )

    def test_fail_on_invalid_role(self):
        self.assert_pcs_fail(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor role=abc"
            ).split(),
            (
                "Error: 'abc' is not a valid role value, use {}\n".format(
                    format_list(const.PCMK_ROLES)
                )
                + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_force_invalid_role(self):
        self.assert_pcs_fail(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor role=abc --force"
            ).split(),
            (
                "Error: 'abc' is not a valid role value, use {}\n".format(
                    format_list(const.PCMK_ROLES)
                )
                + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_on_invalid_on_fail(self):
        self.assert_pcs_fail_regardless_of_force(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor on-fail=Abc"
            ).split(),
            (
                "Error: 'Abc' is not a valid on-fail value, use 'block', "
                "'demote', 'fence', 'ignore', 'restart', 'restart-container', "
                "'standby', 'stop'\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_on_invalid_record_pending(self):
        self.assert_pcs_fail_regardless_of_force(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor record-pending=Abc"
            ).split(),
            (
                "Error: 'Abc' is not a valid record-pending value, use a "
                "pacemaker boolean value: '0', '1', 'false', 'n', 'no', "
                "'off', 'on', 'true', 'y', 'yes'\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_on_invalid_enabled(self):
        self.assert_pcs_fail_regardless_of_force(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor enabled=Abc"
            ).split(),
            (
                "Error: 'Abc' is not a valid enabled value, use a pacemaker "
                "boolean value: '0', '1', 'false', 'n', 'no', 'off', 'on', "
                "'true', 'y', 'yes'\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_on_combination_of_start_delay_and_interval_origin(self):
        self.assert_pcs_fail_regardless_of_force(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor start-delay=10 interval-origin=20"
            ).split(),
            (
                "Error: Only one of resource operation options "
                "'interval-origin' and 'start-delay' can be used\n"
                + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_on_invalid_interval(self):
        self.assert_pcs_fail_regardless_of_force(
            (
                "resource create --no-default-ops R ocf:pcsmock:minimal "
                "op monitor interval="
            ).split(),
            (
                "Error: '' is not a valid interval value, use time interval "
                "(e.g. 1, 2s, 3m, 4h, ...)\n" + ERRORS_HAVE_OCCURRED
            ),
        )


class FailOrWarnGroup(ResourceTest):
    FUTURE = ""
    GROUP = "--group"
    AFTER = "--after"
    BEFORE = "--before"
    DEPRECATED_GROUP = (
        "Deprecation Warning: Using '--group' is deprecated and will be "
        "replaced with 'group' in a future release. Specify --future to switch "
        "to the future behavior.\n"
    )
    DEPRECATED_AFTER = (
        "Deprecation Warning: Using '--after' is deprecated and will be "
        "replaced with 'after' in a future release. Specify --future to switch "
        "to the future behavior.\n"
    )
    DEPRECATED_BEFORE = (
        "Deprecation Warning: Using '--before' is deprecated and will be "
        "replaced with 'before' in a future release. Specify --future to switch "
        "to the future behavior.\n"
    )

    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def test_fail_when_invalid_group(self):
        self.assert_pcs_fail(
            f"resource create R ocf:pcsmock:minimal {self.GROUP} 1 {self.FUTURE}".split(),
            (
                self.DEPRECATED_GROUP
                + "Error: invalid group name '1', '1' is not a valid first character"
                " for a group name\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_when_try_use_id_of_another_element(self):
        self.assert_effect(
            (
                "resource create R1 ocf:pcsmock:minimal --no-default-ops "
                "meta a=b"
            ).split(),
            """<resources>
                <primitive class="ocf" id="R1" provider="pcsmock"
                    type="minimal"
                >
                    <meta_attributes id="R1-meta_attributes">
                        <nvpair id="R1-meta_attributes-a" name="a" value="b"/>
                    </meta_attributes>
                    <operations>
                        <op id="R1-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>""",
            stderr_full=fixture_meta_attributes_warning(
                ["a"], ra_const.PRIMITIVE_META
            ),
        )
        self.assert_pcs_fail(
            (
                "resource create R2 ocf:pcsmock:minimal "
                f"{self.GROUP} R1-meta_attributes {self.FUTURE}"
            ).split(),
            (
                self.DEPRECATED_GROUP
                + "Error: 'R1-meta_attributes' is not a group\n"
                + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_when_entered_both_after_and_before(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:minimal "
                f"{self.GROUP} G {self.AFTER} S1 {self.BEFORE} S2 {self.FUTURE}"
            ).split(),
            (
                self.DEPRECATED_GROUP
                + self.DEPRECATED_BEFORE
                + self.DEPRECATED_AFTER
                + f"Error: you cannot specify both {self.BEFORE} and {self.AFTER}\n"
            ),
        )

    def test_fail_when_after_is_used_without_group(self):
        self.assert_pcs_fail(
            "resource create R ocf:pcsmock:minimal --after S1".split(),
            "Error: you cannot use --after without --group\n",
        )

    def test_fail_when_before_is_used_without_group(self):
        self.assert_pcs_fail(
            "resource create R ocf:pcsmock:minimal --before S1".split(),
            "Error: you cannot use --before without --group\n",
        )

    def test_fail_when_before_after_conflicts_and_moreover_without_group(self):
        self.assert_pcs_fail(
            "resource create R ocf:pcsmock:minimal --after S1 --before S2".split(),
            "Error: you cannot use --before without --group\n",
        )

    def test_fail_when_before_does_not_exist(self):
        self.assert_pcs_success(
            (
                f"resource create R0 ocf:pcsmock:minimal {self.GROUP} G1 "
                f"{self.FUTURE}"
            ).split(),
            stderr_full=self.DEPRECATED_GROUP,
        )
        self.assert_pcs_fail(
            (
                f"resource create R2 ocf:pcsmock:minimal {self.GROUP} G1 "
                f"{self.BEFORE} R1 {self.FUTURE}"
            ).split(),
            (
                self.DEPRECATED_GROUP
                + self.DEPRECATED_BEFORE
                + "Error: 'R1' does not exist\n"
                + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_when_use_before_with_new_group(self):
        self.assert_pcs_fail(
            (
                f"resource create R2 ocf:pcsmock:minimal {self.GROUP} G1 "
                f"{self.BEFORE} R1 {self.FUTURE}"
            ).split(),
            (
                self.DEPRECATED_GROUP
                + self.DEPRECATED_BEFORE
                + "Error: 'R1' does not exist\n"
                + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_when_after_does_not_exist(self):
        self.assert_pcs_success(
            (
                f"resource create R0 ocf:pcsmock:minimal {self.GROUP} G1 "
                f"{self.FUTURE}"
            ).split(),
            stderr_full=self.DEPRECATED_GROUP,
        )
        self.assert_pcs_fail(
            (
                f"resource create R2 ocf:pcsmock:minimal {self.GROUP} G1 "
                f"{self.AFTER} R1 {self.FUTURE}"
            ).split(),
            (
                self.DEPRECATED_GROUP
                + self.DEPRECATED_AFTER
                + "Error: 'R1' does not exist\n"
                + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_when_use_after_with_new_group(self):
        self.assert_pcs_fail(
            (
                f"resource create R2 ocf:pcsmock:minimal {self.GROUP} G1 "
                f"{self.AFTER} R1 {self.FUTURE}"
            ).split(),
            (
                self.DEPRECATED_GROUP
                + self.DEPRECATED_AFTER
                + "Error: 'R1' does not exist\n"
                + ERRORS_HAVE_OCCURRED
            ),
        )


class FailOrWarnGroupFuture(FailOrWarnGroup):
    FUTURE = "--future"
    GROUP = "group"
    AFTER = "after"
    BEFORE = "before"
    DEPRECATED_GROUP = DEPRECATED_AFTER = DEPRECATED_BEFORE = ""

    def test_fail_when_entered_both_after_and_before(self):
        self.assert_pcs_fail(
            (
                "resource create R ocf:pcsmock:minimal "
                f"{self.GROUP} G {self.AFTER} S1 {self.BEFORE} S2 {self.FUTURE}"
            ).split(),
            (
                self.DEPRECATED_GROUP
                + self.DEPRECATED_BEFORE
                + self.DEPRECATED_AFTER
                + f"Error: you cannot specify both '{self.BEFORE}' and '{self.AFTER}'\n"
            ),
        )

    def test_fail_when_after_is_used_without_group(self):
        self.assert_pcs_fail(
            "resource create R ocf:pcsmock:minimal after S1".split(),
            "Error: missing value of 'after' option\n",
        )

    def test_fail_when_before_is_used_without_group(self):
        self.assert_pcs_fail(
            "resource create R ocf:pcsmock:minimal before S1".split(),
            "Error: missing value of 'before' option\n",
        )

    def test_fail_when_before_after_conflicts_and_moreover_without_group(self):
        self.assert_pcs_fail(
            "resource create R ocf:pcsmock:minimal after S1 before S2".split(),
            "Error: missing value of 'after' option\n",
        )


class FailOrWarnPacemakerRemoteOrGuestNode(ResourceTest):
    def setUp(self):
        super().setUp()
        self.pcs_runner.mock_settings = get_mock_settings()

    def test_fail_when_on_pacemaker_remote_attempt(self):
        self.assert_pcs_fail(
            "resource create R2 ocf:pacemaker:remote".split(),
            (
                "Error: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'"
                ", use --force to override\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_warn_when_on_pacemaker_remote_attempt(self):
        self.assert_pcs_success(
            "resource create R2 ocf:pacemaker:remote --force".split(),
            stderr_full=(
                "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
            ),
        )

    def test_fail_when_on_pacemaker_remote_conflict_with_existing_node(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote --force".split(),
            stderr_full=(
                "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
            ),
        )

        self.assert_pcs_fail(
            "resource create R2 ocf:pacemaker:remote server=R --force".split(),
            (
                "Warning: this command is not sufficient for creating a "
                "remote connection, use 'pcs cluster node add-remote'\n"
                "Error: Node address 'R' is already used by existing nodes; "
                "please, use other address\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_when_on_pacemaker_remote_conflict_with_existing_id(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote server=R2 --force".split(),
            stderr_full=(
                "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
            ),
        )

        self.assert_pcs_fail(
            "resource create R2 ocf:pacemaker:remote --force".split(),
            (
                "Warning: this command is not sufficient for creating a "
                "remote connection, use 'pcs cluster node add-remote'\n"
                "Error: Node address 'R2' is already used by existing nodes; "
                "please, use other address\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_when_on_guest_conflict_with_existing_node(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote --force".split(),
            stderr_full=(
                "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
            ),
        )

        self.assert_pcs_fail(
            (
                "resource create R2 ocf:pcsmock:minimal "
                "meta remote-node=R --force"
            ).split(),
            (
                "Warning: this command is not sufficient for creating a "
                "guest node, use 'pcs cluster node add-guest'\n"
                "Error: Cannot set name of the guest node to 'R' because that "
                "ID already exists in the cluster configuration.\n"
                + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_when_on_guest_conflict_with_existing_node_host(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote server=HOST --force".split(),
            stderr_full=(
                "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
            ),
        )

        self.assert_pcs_fail(
            (
                "resource create R2 ocf:pcsmock:minimal "
                "meta remote-node=HOST --force"
            ).split(),
            (
                "Warning: this command is not sufficient for creating a "
                "guest node, use 'pcs cluster node add-guest'\n"
                "Error: Cannot set name of the guest node to 'HOST' because "
                "that ID already exists in the cluster configuration.\n"
                + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_fail_when_on_guest_conflict_with_existing_node_host_addr(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote server=HOST --force".split(),
            stderr_full=(
                "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
            ),
        )

        self.assert_pcs_fail(
            (
                "resource create R2 ocf:pcsmock:minimal "
                "meta remote-node=A remote-addr=HOST --force"
            ).split(),
            (
                "Warning: this command is not sufficient for creating a "
                "guest node, use 'pcs cluster node add-guest'\n"
                "Error: Node address 'HOST' is already used by existing nodes; "
                "please, use other address\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_not_fail_when_on_guest_when_conflict_host_with_name(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote server=HOST --force".split(),
            stderr_full=(
                "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
            ),
        )

        self.assert_pcs_success(
            (
                "resource create R2 ocf:pcsmock:minimal "
                "meta remote-node=HOST remote-addr=R --force"
            ).split(),
            stderr_full=(
                "Warning: this command is not sufficient for creating a guest "
                "node, use 'pcs cluster node add-guest'\n"
            ),
        )

    def test_fail_when_on_pacemaker_remote_guest_attempt(self):
        self.assert_pcs_fail(
            "resource create R2 ocf:pcsmock:minimal meta remote-node=HOST".split(),
            (
                "Error: this command is not sufficient for creating a guest "
                "node, use 'pcs cluster node add-guest', use --force to "
                "override\n" + ERRORS_HAVE_OCCURRED
            ),
        )

    def test_warn_when_on_pacemaker_remote_guest_attempt(self):
        self.assert_pcs_success(
            (
                "resource create R2 ocf:pcsmock:minimal "
                "meta remote-node=HOST --force"
            ).split(),
            stderr_full=(
                "Warning: this command is not sufficient for creating a guest node,"
                " use 'pcs cluster node add-guest'\n"
            ),
        )



import contextlib
import os
from typing import Any, Optional, cast

from pcs import (
    settings,
    utils,
)
from pcs.cli.common.errors import CmdLineInputError
from pcs.cli.common.parse_args import Argv, InputModifiers
from pcs.cli.file import metadata as cli_metadata
from pcs.cli.reports.output import error
from pcs.cli.reports.processor import ReportProcessorToConsole
from pcs.common.auth import HostAuthData
from pcs.common.communication.logger import CommunicatorLogger
from pcs.common.file import RawFile, RawFileError
from pcs.common.file_type_codes import PCS_KNOWN_HOSTS
from pcs.common.host import Destination, PcsKnownHost
from pcs.common.node_communicator import Communicator, NodeCommunicatorFactory
from pcs.common.reports.processor import ReportProcessorToLog
from pcs.common.tools import format_os_error
from pcs.common.validate import is_port_number
from pcs.daemon import log
from pcs.lib.communication.nodes import Auth
from pcs.lib.communication.tools import run
from pcs.lib.file import toolbox
from pcs.lib.file.instance import FileInstance
from pcs.lib.file.raw_file import raw_file_error_report
from pcs.lib.host.config.facade import Facade
from pcs.lib.interface.config import ParserErrorException


def local_auth_cmd(lib: Any, argv: Argv, modifiers: InputModifiers) -> None:
    """
    Options:
      * -u - username
      * -p - password
      * --request-timeout - timeout for HTTP requests
    """
    if os.geteuid() == 0:
        raise error("This command cannot be run as superuser")

    del lib
    modifiers.ensure_only_supported("-u", "-p", "--request-timeout")
    if len(argv) > 1:
        raise CmdLineInputError()
    port = argv[0] if argv else settings.pcsd_default_port
    if not is_port_number(str(port)):
        raise CmdLineInputError()

    username, password = utils.get_user_and_pass()
    LOCALHOST = "localhost"

    report_processor = ReportProcessorToConsole(bool(modifiers.get("--debug")))
    node_communicator = _get_node_communicator(
        # we know that --request-timeout value is either not defined, or it is
        # a valid number, since it is validated in app.py, so this is safe
        int(modifiers.get("--request-timeout") or 0)
        if modifiers.is_specified("--request-timeout")
        else None
    )

    com_cmd = Auth(
        {
            LOCALHOST: HostAuthData(
                username=username,
                password=password,
                dest_list=[Destination(LOCALHOST, int(port))],
            )
        },
        report_processor,
    )
    received_tokens: dict[str, str] = run(node_communicator, com_cmd)  # type: ignore[no-untyped-call]
    if LOCALHOST not in received_tokens:
        # errors have already been reported by the communication command
        raise SystemExit(1)

    new_known_hosts = [
        PcsKnownHost(
            name=LOCALHOST,
            token=received_tokens[LOCALHOST],
            dest_list=[Destination(LOCALHOST, int(port))],
        )
    ]

    file_instance = FileInstance(
        RawFile(cli_metadata.for_file_type(PCS_KNOWN_HOSTS)),  # type: ignore[no-untyped-call]
        toolbox.for_file_type(PCS_KNOWN_HOSTS),
    )
    try:
        if not file_instance.raw_file.exists():
            known_hosts_facade = Facade.create()
        else:
            known_hosts_facade = cast(Facade, file_instance.read_to_facade())
    except RawFileError as e:
        report_processor.report(raw_file_error_report(e))
    except ParserErrorException as e:
        report_processor.report_list(
            file_instance.parser_exception_to_report_list(e)
        )
    if report_processor.has_errors:
        raise SystemExit(1)

    known_hosts_facade.update_known_hosts(new_known_hosts)

    # create the .pcs folder in home
    try:
        path = os.path.expanduser("~/.pcs")
        with contextlib.suppress(FileExistsError):
            # its not an error if the folder already exists
            os.mkdir(path, 0o700)
        # documentation states that "mode" in os.mkdir is ignored on some
        # platforms and chmod should be used
        os.chmod(path, 0o700)
    except OSError as e:
        raise error(format_os_error(e)) from e

    try:
        file_instance.write_facade(known_hosts_facade, can_overwrite=True)
    except RawFileError as e:
        report_processor.report(raw_file_error_report(e))
    if report_processor.has_errors:
        raise SystemExit(1)


# used to simplify testing
def _get_node_communicator(timeout: Optional[int]) -> Communicator:
    log_report_processor = ReportProcessorToLog(log.pcsd)
    return NodeCommunicatorFactory(
        CommunicatorLogger([log_report_processor]),
        user=None,
        groups=None,
        request_timeout=timeout,
    ).get_communicator()



from unittest import TestCase

from pcs.lib.commands.remote_node import get_resource_ids

from pcs_test.tools.command_env import get_env_tools


class GetResourceIdsFromRemoteNodeIdentifier(TestCase):
    NODE_NAME = "A"
    NODE_NAME_MULTIPLE = "B"

    def setUp(self):
        self.env_assist, self.config = get_env_tools(self)
        self.config.runner.cib.load(
            resources=f"""
                <resources>
                    <primitive class="ocf" id="{self.NODE_NAME}" provider="pacemaker" type="remote" />
                    <primitive class="ocf" id="{self.NODE_NAME_MULTIPLE}" provider="pacemaker" type="remote">
                        <instance_attributes id="node-name-instance_attributes">
                            <nvpair
                                id="node-name-instance_attributes-server"
                                name="server" value="foo"
                            />
                        </instance_attributes>
                    </primitive>
                    <primitive class="ocf" id="X" provider="pacemaker" type="remote">
                        <instance_attributes id="node-name-instance_attributes">
                            <nvpair
                                id="node-name-instance_attributes-server"
                                name="server" value="{self.NODE_NAME_MULTIPLE}"
                            />
                        </instance_attributes>
                    </primitive>
                </resources>
            """
        )

    def test_node_not_found(self):
        resource_ids = get_resource_ids(
            self.env_assist.get_env(), "nonexistent"
        )
        self.assertEqual(resource_ids, [])

    def test_node_found(self):
        resource_ids = get_resource_ids(
            self.env_assist.get_env(), self.NODE_NAME
        )
        self.assertEqual(resource_ids, [self.NODE_NAME])

    def test_multiple_nodes_found(self):
        resource_ids = get_resource_ids(
            self.env_assist.get_env(), self.NODE_NAME_MULTIPLE
        )
        self.assertEqual(resource_ids, [self.NODE_NAME_MULTIPLE, "X"])

    def test_non_remote_resource(self):
        self.config.runner.cib.load(
            instead="runner.cib.load",
            resources=f"""
                <resources>
                    <primitive class="ocf" id="{self.NODE_NAME_MULTIPLE}" provider="pacemaker" type="Dummy" />
                    <primitive class="ocf" id="X" provider="pacemaker" type="Dummy">
                        <instance_attributes id="node-name-instance_attributes">
                            <nvpair
                                id="node-name-instance_attributes-server"
                                name="server" value="{self.NODE_NAME_MULTIPLE}"
                            />
                        </instance_attributes>
                    </primitive>
                </resources>
            """,
        )
        resource_ids = get_resource_ids(
            self.env_assist.get_env(), self.NODE_NAME_MULTIPLE
        )
        self.assertEqual(resource_ids, [])
